#include "w5500.h"
#include "socket.h"
#include "loopback.h"
#include <stdio.h>

////////////////////////////////////////////////
// Shared Buffer Definition for LOOPBACK TEST //
////////////////////////////////////////////////
#define DATA_BUF_SIZE   2048
uint8_t gDATABUF[DATA_BUF_SIZE];

int32_t loopback_tcps(uint8_t sn, uint8_t* buf, uint16_t port)
{
	   int32_t ret;
	   uint16_t size = 0, sentsize=0;
	   switch(getSn_SR(sn))
	   {
	      case SOCK_ESTABLISHED :
	         if(getSn_IR(sn) & Sn_IR_CON)
	         {
	            printf("%d:Connected\r\n",sn);
	            setSn_IR(sn,Sn_IR_CON);
	         }
	         if((size = getSn_RX_RSR(sn)) > 0)
	         {
	            if(size > DATA_BUF_SIZE) size = DATA_BUF_SIZE;
	            ret = recv(sn,buf,size);
	            if(ret <= 0) return ret;
	            sentsize = 0;
	            while(size != sentsize)
	            {
	               ret = send(sn,buf+sentsize,size-sentsize);
	               if(ret < 0)
	               {
	                  close(sn);
	                  return ret;
	               }
	               sentsize += ret; // Don't care SOCKERR_BUSY, because it is zero.
	            }
	         }
	         break;
	      case SOCK_CLOSE_WAIT :
	         printf("%d:CloseWait\r\n",sn);
	         if((ret=disconnect(sn)) != SOCK_OK) return ret;
	         printf("%d:Closed\r\n",sn);
	         break;
	      case SOCK_INIT :
	    	  printf("%d:Listen, port [%d]\r\n",sn, port);
	         if( (ret = listen(sn)) != SOCK_OK) return ret;
	         break;
	      case SOCK_CLOSED:
	         printf("%d:LBTStart\r\n",sn);
	         if((ret=socket(sn,Sn_MR_TCP,port,0x00)) != sn)
	            return ret;
	         printf("%d:Opened\r\n",sn);
	         break;
	      default:
	         break;
	   }
	   return 1;
}
#if 0
void loopback_tcpc(SOCKET s, uint16_t port)
{
        uint16_t RSR_len;
        uint16_t received_len;
	uint8_t * data_buf = TX_BUF;

   switch (getSn_SR(s))
   {
   case SOCK_ESTABLISHED:                 /* if connection is established */
      if(ch_status[s]==1)
      {
         printf("\r\n%d : Connected",s);
         ch_status[s] = 2;
      }

      if ((RSR_len = getSn_RX_RSR(s)) > 0)         /* check Rx data */
      {
         if (RSR_len > TX_RX_MAX_BUF_SIZE) RSR_len = TX_RX_MAX_BUF_SIZE;   /* if Rx data size is lager than TX_RX_MAX_BUF_SIZE */
                                                                            /* the data size to read is MAX_BUF_SIZE. */
         received_len = recv(s, data_buf, RSR_len);         /* read the received data */
         send(s, data_buf, received_len);         /* sent the received data */
      }

      break;
   case SOCK_CLOSE_WAIT:                                 /* If the client request to close */
      printf("\r\n%d : CLOSE_WAIT", s);
      if ((RSR_len = getSn_RX_RSR(s)) > 0)         /* check Rx data */
      {
         if (RSR_len > TX_RX_MAX_BUF_SIZE) RSR_len = TX_RX_MAX_BUF_SIZE;   /* if Rx data size is lager than TX_RX_MAX_BUF_SIZE */
                                                                                    /* the data size to read is MAX_BUF_SIZE. */
         received_len = recv(s, data_buf, RSR_len);         /* read the received data */
      }
      disconnect(s);
      ch_status[s] = 0;
      break;
   case SOCK_CLOSED:                                               /* if a socket is closed */
      if(!ch_status[s])
      {
         printf("\r\n%d : Loop-Back TCP Client Started. port: %d", s, port);
         ch_status[s] = 1;
      }
      if(socket(s, Sn_MR_TCP, any_port++, 0x00) == 0)    /* reinitialize the socket */
      {
         printf("\a%d : Fail to create socket.",s);
         ch_status[s] = 0;
      }
      break;
        case SOCK_INIT:     /* if a socket is initiated */
                if(time_return() - presentTime >= (tick_second * 3)) {  /* For TCP client's connection request delay : 3 sec */
                        connect(s, Chconfig_Type_Def.destip, Chconfig_Type_Def.port); /* Try to connect to TCP server(Socket, DestIP, DestPort) */
                        presentTime = time_return();
                }
                break;
        default:
                break;
        }
}
#endif

int32_t loopback_udps(uint8_t sn, uint8_t* buf, uint16_t port)
{
	   int32_t  ret;
	   uint16_t size, sentsize;
	   uint8_t  destip[4];
	   uint16_t destport;
	   //uint8_t  packinfo = 0;
	   switch(getSn_SR(sn))
	   {
	      case SOCK_UDP :
	         if((size = getSn_RX_RSR(sn)) > 0)
	         {
	            if(size > DATA_BUF_SIZE) size = DATA_BUF_SIZE;
	            ret = recvfrom(sn,buf,size,destip,(uint16_t*)&destport);
	            if(ret <= 0)
	            {
	               printf("%d: recvfrom error. %ld\r\n",sn,ret);
	               return ret;
	            }
	            size = (uint16_t) ret;
	            sentsize = 0;
	            while(sentsize != size)
	            {
	               ret = sendto(sn,buf+sentsize,size-sentsize,destip,destport);
	               if(ret < 0)
	               {
	                  printf("%d: sendto error. %ld\r\n",sn,ret);
	                  return ret;
	               }
	               sentsize += ret; // Don't care SOCKERR_BUSY, because it is zero.
	            }
	         }
	         break;
	      case SOCK_CLOSED:
	         printf("%d:LBUStart\r\n",sn);
	         if((ret=socket(sn,Sn_MR_UDP,port,0x00)) != sn)
	            return ret;
	         printf("%d:Opened, port [%d]\r\n",sn, port);
	         break;
	      default :
	         break;
	   }
	   return 1;
}
//////////////////////////////////////////////////////////////////////////////
