#include "main.h"

/***************************************
 * SOCKET NUMBER DEFINION for Examples *
 ***************************************/
#define SOCK_DHCP			0
#define MY_MAX_DHCP_RETRY	3

/**********************************************
 * Shared Buffer Definition for LOOPBACK TEST *
 **********************************************/
#define DATA_BUF_SIZE   2048
uint8_t gDATABUF[DATA_BUF_SIZE];

/**************************************************************************//**
 * @brief Default Network Inforamtion
 *****************************************************************************/
uint8_t Enable_DHCP = OFF;
NetInfo gNETINFO = { .mac = {0x00, 0x08, 0xdc,0x1d, 0x64, 0x3d},
                            .ip = {192, 168, 0, 123},
                            .sn = {255, 255, 255, 0},
                            .gw = {192, 168, 0, 1},
                            .dns = {168, 126, 63, 1},
//                            .dns = {0, 0, 0, 0},
                            .dhcp = NETINFO_STATIC };

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
	if (TimingDelay != 0x00)
	{
		TimingDelay--;
	}

//	if (Timer2_Counter++ > 1000) { // 1m x 1000 = 1sec
//		Timer2_Counter = 0;
//	}
}

/* W5500 configuration */
void W5500_Network_Configuration(void)
{
	uint8_t tmp;

    /* PHY link status check */
    do
    {
       if(ctlwizchip(CW_GET_PHYLINK, (void*)&tmp) == -1)
          printf("Unknown PHY Link stauts.\r\n");
    }while(tmp == PHY_LINK_OFF);

	DHCP_Configuration();
}

void SysTick_Init(void)
{
	if(SysTick_Config(SystemCoreClock/10000))	 //1ms
	{
		while(1);
	}
}

void Delay_ms(uint16_t nTime)
{
	TimingDelay = nTime;

	while(TimingDelay !=0);
}

void W5500_Reset()
{
	GPIO_ResetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
	Delay_ms(1);    /*delay 1000ms systick 1ms */
	GPIO_SetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
	Delay_ms(1500);
}

/******************************************************************************
 * @brief  Network Init
 * Intialize the network information to be used in WIZCHIP
 *****************************************************************************/
void network_init(void)
{
	uint8_t tmpstr[6] = {0,};
	NetInfo netinfo;

	// Set Network information from netinfo structure
	ctlnetwork(CN_SET_NETINFO, (void*)&gNETINFO);

#ifdef _MAIN_DEBUG_
	// Get Network information
	ctlnetwork(CN_GET_NETINFO, (void*)&netinfo);

	// Display Network Information
	ctlwizchip(CW_GET_ID,(void*)tmpstr);

	if(netinfo.dhcp == NETINFO_DHCP) printf("\r\n=== %s NET CONF : DHCP ===\r\n",(char*)tmpstr);
	else printf("\r\n=== %s NET CONF : Static ===\r\n",(char*)tmpstr);

	printf("MAC: %02X:%02X:%02X:%02X:%02X:%02X\r\n",netinfo.mac[0],netinfo.mac[1],netinfo.mac[2],
			netinfo.mac[3],netinfo.mac[4],netinfo.mac[5]);
	printf("SIP: %d.%d.%d.%d\r\n", netinfo.ip[0],netinfo.ip[1],netinfo.ip[2],netinfo.ip[3]);
	printf("GAR: %d.%d.%d.%d\r\n", netinfo.gw[0],netinfo.gw[1],netinfo.gw[2],netinfo.gw[3]);
	printf("SUB: %d.%d.%d.%d\r\n", netinfo.sn[0],netinfo.sn[1],netinfo.sn[2],netinfo.sn[3]);
	printf("DNS: %d.%d.%d.%d\r\n", netinfo.dns[0],netinfo.dns[1],netinfo.dns[2],netinfo.dns[3]);
	printf("===========================\r\n");
#endif
}

void DHCP_Configuration(void)
{
	uint8_t my_dhcp_retry = 0;

	if (Enable_DHCP == OFF)
	{
		network_init();
		if (ping_auto(0, gNETINFO.ip) == 1)
			printf("ping SUCCESS\n");
		else
			printf("ping Fail\n");
		Main_Menu();
	}
	else
	{
	    // must be set the default mac before DHCP started.
		setSHAR(gNETINFO.mac);

		DHCP_init(SOCK_DHCP, gDATABUF);
		// if you want defiffent action instead defalut ip assign,update, conflict,
		// if cbfunc == 0, act as default.
		reg_dhcp_cbfunc(my_ip_assign, my_ip_assign, my_ip_conflict);

	    /* Main Loop */
	    while(1)
	    {
			switch(DHCP_run())
			{
				case DHCP_IP_ASSIGN:
				case DHCP_IP_CHANGED:
					/* If this block empty, act with default_ip_assign & default_ip_update */
					//
					// This example calls my_ip_assign in the two case.
					//
					// Add to ...
					//
					Main_Menu();
					break;
				case DHCP_IP_LEASED:
					//
					// TO DO YOUR NETWORK APPs.
					//
					break;
				case DHCP_FAILED:
					/* ===== Example pseudo code =====  */
					// The below code can be replaced your code or omitted.
					// if omitted, retry to process DHCP
					my_dhcp_retry++;
					if(my_dhcp_retry > MY_MAX_DHCP_RETRY)
					{
						#ifdef _MAIN_DEBUG_
						printf(">> DHCP %d Failed\r\n", my_dhcp_retry);
						#endif
						my_dhcp_retry = 0;
						DHCP_stop();      // if restart, recall DHCP_init()
						network_init();   // apply the default static network and print out netinfo to serial
					}
					break;
				default:
					break;
			}
	    }
	}
}

/*   SPI Initialization  */
void SPI_Configuration(void)
{
	GPIO_InitTypeDef 		GPIO_InitStructure;
	SPI_InitTypeDef   		SPI_InitStructure;

	/* Enable SPI1 and GPIOA clocks */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_W5500_SCS, ENABLE);

	/* Set Chip Select pin */
	GPIO_InitStructure.GPIO_Pin = GPIO_W5500_SCS_Pin;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_Init(GPIO_W5500_SCS_PORT, &GPIO_InitStructure);
	GPIO_SetBits(GPIO_W5500_SCS_PORT, GPIO_W5500_SCS_Pin);

  	/* Configure SPI1 pins: SCK, MISO and MOSI -------------*/
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);

//	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
//   	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
//   	GPIO_Init(GPIOA, &GPIO_InitStructure);



	// jhpark. add.
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource15, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);

	/* Set SPI interface */
	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_InitStructure.SPI_CRCPolynomial = 7;

	SPI_Init(SPI1,&SPI_InitStructure);

	SPI_Cmd(SPI1,ENABLE);					//Enable  SPI1
}

/* IO port Configuration */
void W5500_GPIO_Configuration(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	/* Define Reset W5500 output */

	RCC_APB2PeriphClockCmd( RCC_W5500_RST, ENABLE);
	GPIO_InitStructure.GPIO_Pin  = GPIO_W5500_RST_Pin;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_Init(GPIO_W5500_RST_PORT, &GPIO_InitStructure);
//	GPIO_ResetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
	// jhpark. add.
	GPIO_SetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
//	GPIO_ResetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
//	GPIO_SetBits(GPIO_W5500_RST_PORT, GPIO_W5500_RST_Pin);
}

void USART1_Init(void)
{
	USART_InitTypeDef USART_InitStructure;

	/* USARTx configuration ------------------------------------------------------*/
	/* USARTx configured as follow:
		- BaudRate = 115200 baud
		- Word Length = 8 Bits
		- One Stop Bit
		- No parity
		- Hardware flow control disabled (RTS and CTS signals)
		- Receive and transmit enabled
	*/
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No ;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	/* Configure the USARTx */
	USART_Init(USART1, &USART_InitStructure);

	/* Enable the USARTx */
	USART_Cmd(USART1, ENABLE);

	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
}

void USART1_GPIO_Configuration(void)
{
	GPIO_InitTypeDef 		GPIO_InitStructure;

	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

	// Configure the GPIO ports( USART1 Transmit and Receive Lines)
	// Configure the USART1_Tx as Alternate function Push-Pull
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_USART1);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_USART1);

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Pin =  USART1_TX;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Configure the USART1_Rx as input floating
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Pin = USART1_RX;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void NVIC_Configuration(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Configure one bit for preemption priority */
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);

	/* Enable the USART1 Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

/* system Configuration */
void System_Initialization(void)
{

	NVIC_Configuration();

	/* SPI Configuration */
	SPI_Configuration();

	/* IO Configuration */
	W5500_GPIO_Configuration();

	/* LED GPIO Configuration */
	LED_GPIO_Config();

	/* USART Configuration */
	USART1_GPIO_Configuration();
	USART1_Init();
}

/*******************************************************
 * @ brief Call back for ip assing & ip update from DHCP
 *******************************************************/
void my_ip_assign(void)
{
   getIPfromDHCP(gNETINFO.ip);
   getGWfromDHCP(gNETINFO.gw);
   getSNfromDHCP(gNETINFO.sn);
   getDNSfromDHCP(gNETINFO.dns);
   gNETINFO.dhcp = NETINFO_DHCP;
   /* Network initialization */
   network_init();      // apply from dhcp
#ifdef _MAIN_DEBUG_
   printf("DHCP LEASED TIME : %ld Sec.\r\n", getDHCPLeasetime());
#endif
}

/************************************
 * @ brief Call back for ip Conflict
 ************************************/
void my_ip_conflict(void)
{
#ifdef _MAIN_DEBUG_
	printf("CONFLICT IP from DHCP\r\n");
#endif
   //halt or reset or any...
   while(1); // this example is halt.
}

/*****************************************************************
                           Main Program
*****************************************************************/
int main(void)
{
	uint8_t memsize[2][8] = { {2, 2, 2, 2, 2, 2, 2, 2}, {2, 2, 2, 2, 2, 2, 2, 2} };

	/* Initialize STM32F103 */
	System_Initialization();

	printf("Main Program Start!!\n");

	SysTick_Init();

#if   _WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_SPI_VDM_
    reg_wizchip_cs_cbfunc(wizchip_select, wizchip_deselect);
#else
    reg_wizchip_cs_cbfunc(wizchip_select, wizchip_deselect);
#endif

    /* SPI Read & Write callback function */
    reg_wizchip_spi_cbfunc(wizchip_read, wizchip_write);

	Delay_ms(1000);
    /* wizchip initialize*/
    if(ctlwizchip(CW_INIT_WIZCHIP, (void*)memsize) == -1)
    {
       printf("WIZCHIP Initialized fail.\r\n");
       while(1);
    }

    printf("chip Version = 0x%x\n", getVERSIONR());

	/* Config W5500 */
	W5500_Network_Configuration();

	while(1){}
}

void  wizchip_select(void)
{
	W5500_SCS_LOW;
}

void  wizchip_deselect(void)
{
	W5500_SCS_HIGH;
}

void  wizchip_write(uint8_t wb)
{
//   xSPISingleDataReadWrite(WIZCHIP_SPI_BASE,wb);
	SPI_I2S_SendData(SPI1, wb);
}

uint8_t wizchip_read()
{
//   return xSPISingleDataReadWrite(WIZCHIP_SPI_BASE,0xFF);

	return SPI_I2S_ReceiveData(SPI1);
}
